/*
** main.c
**
** Project: 	interrupt-usage-multi
**
** Date:    	12/03/2015
** Revision:
** Author:  	DC
**
** Comments:    Timer 2-driven ISR used to toggle LED LD1 every 100mS
**              Timer 4-driven ISR used to toggle LED LD4 every 500mS
**
** Hardware Design Reference:
**  "chipKIT WiFire Rev C"
**
** Dependencies:
** 
**
** User Guide:
**
**
*/

/************************************************************************

SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP TECHNOLOGY INC OR ITS LICENSORS BE LIABLE OR OBLIGATED
UNDER CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY,
OR OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

**************************************************************************/

/** INCLUDES ******************************************************************/

#include <xc.h>             /* contains Vector Name/Number Macros */
#include <sys/attribs.h>    /* contains __ISR() Macros */

/** CONFIGURATION *************************************************************/

// Key Settings:
// OSCILLATOR: 24MHz EC Oscillator w. PLL
// SYSCLK = 200 MHz (set in config bits), PBxCLK = 8 MHz (set in application code)
// JTAG PORT: Disabled
// WATCHDOG TIMER: Disabled
// DEBUG/PGM PINS: PGEC2/PGED2

// DEVCFG3
// USERID = No Setting
#pragma config FMIIEN = OFF             // Ethernet RMII/MII Enable (RMII Enabled)
#pragma config FETHIO = ON              // Ethernet I/O Pin Select (Default Ethernet I/O)
#pragma config PGL1WAY = ON             // Permission Group Lock One Way Configuration (Allow only one reconfiguration)
#pragma config PMDL1WAY = ON            // Peripheral Module Disable Configuration (Allow only one reconfiguration)
#pragma config IOL1WAY = OFF            // Peripheral Pin Select Configuration (Allow multiple reconfigurations)
#pragma config FUSBIDIO = ON            // USB USBID Selection (Controlled by the USB Module)

// DEVCFG2
#pragma config FPLLIDIV = DIV_3         // System PLL Input Divider (3x Divider)
#pragma config FPLLRNG = RANGE_5_10_MHZ // System PLL Input Range (5-10 MHz Input)
#pragma config FPLLICLK = PLL_POSC      // System PLL Input Clock Selection (POSC is input to the System PLL)
#pragma config FPLLMULT = MUL_50        // System PLL Multiplier (PLL Multiply by 50)
#pragma config FPLLODIV = DIV_2         // System PLL Output Clock Divider (2x Divider)
#pragma config UPLLFSEL = FREQ_24MHZ    // USB PLL Input Frequency Selection (USB PLL input is 24 MHz)

// DEVCFG1
#pragma config FNOSC = SPLL             // Oscillator Selection Bits (System PLL)
#pragma config DMTINTV = WIN_127_128    // DMT Count Window Interval (Window/Interval value is 127/128 counter value)
#pragma config FSOSCEN = OFF            // Secondary Oscillator Enable (Disable SOSC)
#pragma config IESO = ON                // Internal/External Switch Over (Enabled)
#pragma config POSCMOD = EC             // Primary Oscillator Configuration (External clock mode)
#pragma config OSCIOFNC = OFF           // CLKO Output Signal Active on the OSCO Pin (Disabled)
#pragma config FCKSM = CSECME           // Clock Switching and Monitor Selection (Clock Switch Enabled, FSCM Enabled)
#pragma config WDTPS = PS1048576        // Watchdog Timer Postscaler (1:1048576)
#pragma config WDTSPGM = STOP           // Watchdog Timer Stop During Flash Programming (WDT stops during Flash programming)
#pragma config WINDIS = NORMAL          // Watchdog Timer Window Mode (Watchdog Timer is in non-Window mode)
#pragma config FWDTEN = OFF             // Watchdog Timer Enable (WDT Disabled)
#pragma config FWDTWINSZ = WINSZ_25     // Watchdog Timer Window Size (Window size is 25%)
#pragma config DMTCNT = DMT31           // Deadman Timer Count Selection (2^31 (2147483648))
#pragma config FDMTEN = OFF             // Deadman Timer Enable (Deadman Timer is disabled)

// DEVCFG0
#pragma config DEBUG = OFF              // Background Debugger Enable (Debugger is disabled)
#pragma config JTAGEN = OFF             // JTAG Enable (JTAG Disabled)
#pragma config ICESEL = ICS_PGx2        // ICE/ICD Comm Channel Select (Communicate on PGEC2/PGED2)
#pragma config TRCEN = ON               // Trace Enable (Trace features in the CPU are enabled)
#pragma config BOOTISA = MIPS32         // Boot ISA Selection (Boot code and Exception code is MIPS32)
#pragma config FECCCON = OFF_UNLOCKED   // Dynamic Flash ECC Configuration (ECC and Dynamic ECC are disabled (ECCCON bits are writable))
#pragma config FSLEEP = OFF             // Flash Sleep Mode (Flash is powered down when the device is in Sleep mode)
#pragma config DBGPER = ALLOW_PG2       // Debug Mode CPU Access Permission (Allow CPU access to Permission Group 2 permission regions)
#pragma config SMCLR = MCLR_NORM        // Soft Master Clear Enable bit (MCLR pin generates a normal system Reset)
#pragma config SOSCGAIN = GAIN_2X       // Secondary Oscillator Gain Control bits (2x gain setting)
#pragma config SOSCBOOST = ON           // Secondary Oscillator Boost Kick Start Enable bit (Boost the kick start of the oscillator)
#pragma config POSCGAIN = GAIN_2X       // Primary Oscillator Gain Control bits (2x gain setting)
#pragma config POSCBOOST = ON           // Primary Oscillator Boost Kick Start Enable bit (Boost the kick start of the oscillator)
#pragma config EJTAGBEN = NORMAL        // EJTAG Boot (Normal EJTAG functionality)

// DEVCP0
#pragma config CP = OFF                 // Code Protect (Protection Disabled)

/** VARIABLES *****************************************************************/


/** LOCAL MACROS **************************************************************/

#define SYS_CLK_FREQUENCY  (200000000ull)       // Fsys = 200 MHz
#define PB3_CLK_FREQUENCY SYS_CLK_FREQUENCY/8   // Fpb3 = 8 MHz

// LED_LD1 (RG6)
#define LED_LD1       	LATGbits.LATG6
#define TRIS_LED_LD1  	TRISGbits.TRISG6
#define LED_LD1_SET()   LATGSET = _LATG_LATG6_MASK;
#define LED_LD1_CLR()   LATGCLR = _LATG_LATG6_MASK;
#define LED_LD1_INV()   LATGINV = _LATG_LATG6_MASK;

// LED_LD4 (RG15)
#define LED_LD4       	LATGbits.LATG15
#define TRIS_LED_LD4  	TRISGbits.TRISG15
#define LED_LD4_SET()   LATGSET = _LATG_LATG15_MASK;
#define LED_LD4_CLR()   LATGCLR = _LATG_LATG15_MASK;
#define LED_LD4_INV()   LATGINV = _LATG_LATG15_MASK;

/** LOCAL PROTOTYPES ********************************************************/

void InitializeSystem(void);            // Initialize hardware and global variables

/** main() ********************************************************************/

int main(void)
{
    InitializeSystem();

    while(1);
    
} // main()

/******************************************************************************
 * Function:        void InitializeSystem(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This routine takes care of all of the system
 *                  initialization that is required.
 *
 * Note:
 *
 *****************************************************************************/
void InitializeSystem(void)
{
    // PIC32MZ CPU Speed Optimizations (Cache/Wait States/Peripheral Bus Clks)
    // On reset, I+D cache is enabled for max performace setting (write-back with write allocation)
    // No wait setting required for main data RAM
    // Prefetch-cache: Enable prefetch for PFM (any PFM instructions or data)
    PRECONbits.PREFEN = 3;
    // Flash PM Wait States: MZ Flash runs at 2 wait states @ 200 MHz
    PRECONbits.PFMWS = 2;
    // PBClk3 set to 8 MHz (requires config bits setting: Fsys=200MHz)
    // Unlock Sequence
    SYSKEY = 0xAA996655;
    SYSKEY = 0x556699AA;
    // Modify PB3DIV
    PB3DIVbits.PBDIV = 24;   // Fpb3=Fsys/25 (200/25=8MHz)
    // Lock sequence
    SYSKEY = 0x33333333;

    /* Initialize LED LD1 */
    LED_LD1 = 0;
    TRIS_LED_LD1 = 0;

    /* Initialize LED LD4 */
    LED_LD4 = 0;
    TRIS_LED_LD4 = 0;

    /* Initialize Timer 2 Peripheral Settings */
    // Turn off the timer
    T2CONbits.TON = 0;
    // Pre-Scale = 1:256 (T2Clk: 31250Hz)
    T2CONbits.TCKPS = 7;
    // Set T2 period = 100mS
    PR2 = 3125;
    // Clear counter
    TMR2 = 0;

    /* Initialize Timer 2 Interrupt Controller Settings */
    // Set the interrupt priority to 4
    IPC2bits.T2IP = 4;
    // Reset the Timer 2 interrupt flag
    IFS0bits.T2IF = 0;
    // Enable interrupts from Timer 2
    IEC0bits.T2IE = 1;

    /* Initialize Timer 4 Peripheral Settings */
    // Turn off the timer
    T4CONbits.TON = 0;
    // Pre-Scale = 1:256 (T4Clk: 31250Hz)
    T4CONbits.TCKPS = 7;
    // Set T4 period = 500mS
    PR4 = 15625;
    // Clear counter
    TMR4 = 0;

    /* Initialize Timer 4 Interrupt Controller Settings */
    // Set the interrupt priority to 3
    IPC4bits.T4IP = 3;
    // Reset the Timer 4 interrupt flag
    IFS0bits.T4IF = 0;
    // Enable interrupts from Timer 4
    IEC0bits.T4IE = 1;
  
    /* Assign PIC32MZ shadow register sets to specific CPU IPLs */
    PRISS = 0x76543210;
    
    /* Set Interrupt Controller for multi-vector mode */
    INTCONSET = _INTCON_MVEC_MASK;

    /* Enable Interrupt Exceptions */
    // set the CP0 status IE bit high to turn on interrupts globally
    __builtin_enable_interrupts();

    /* Enable the peripherals */
    T2CONbits.TON = 1;
    T4CONbits.TON = 1;

} // InitializeSystem()

/*****************************************************************************
  Function:
	void __ISR_AT_VECTOR (_TIMER_2_VECTOR, IPL4SRS) T2Interrupt(void);

  Description:
	Toggles LED LD1 when the Timer 2 interrupt occurs.

  Precondition:
        Shadow register sets must be assigned to an IPL (PRISS = 0x76543210;)

  Parameters:
	None

  Returns:
  	None
  ***************************************************************************/

void __ISR_AT_VECTOR (_TIMER_2_VECTOR, IPL4SRS) T2Interrupt(void)
{
	// Toggle LED LD1
	LED_LD1_INV();

	// Reset interrupt flag
	IFS0bits.T2IF = 0;
}

/*****************************************************************************
  Function:
	void __ISR_AT_VECTOR (_TIMER_4_VECTOR, IPL3SRS) T4Interrupt(void);

  Description:
	Toggles LED LD4 when the Timer 4 interrupt occurs.

  Precondition:
        Shadow register sets must be assigned to an IPL (PRISS = 0x76543210;)

  Parameters:
	None

  Returns:
  	None
  ***************************************************************************/

void __ISR_AT_VECTOR (_TIMER_4_VECTOR, IPL3SRS) T4Interrupt(void)
{
	// Toggle LED LD4
	LED_LD4_INV();

	// Reset interrupt flag
	IFS0bits.T4IF = 0;
}